// Generated by CoffeeScript 1.12.7
(function() {
  var APIBase, NodeCache, async, bunyan, deferred, jp, log, oauth, request, startsWith, ud;

  jp = require('JSONPath');

  request = require('request');

  async = require('async');

  bunyan = require('bunyan');

  ud = require('underscore');

  NodeCache = require('node-cache');

  log = bunyan.createLogger({
    name: "apibase-client"
  });

  startsWith = require('underscore.string/startsWith');

  deferred = require('deferred');

  oauth = {};

  oauth['bearer'] = function(token, req) {
    if (!token) {
      return req;
    }
    if (!req.headers) {
      req.headers = {};
    }
    req.headers['Authorization'] = 'Bearer ' + token.access_token;
    return req;
  };

  oauth['pivotal tracker'] = function(token, req) {
    if (!req.headers) {
      req.headers = {};
    }
    req.headers['X-TrackerToken'] = token.access_token;
    return req;
  };

  oauth['github'] = oauth.bearer;

  oauth['gdrive'] = function(token, req) {
    req.qs['key'] = require('../config.json').gapi.id;
    return oauth['bearer'](token, req);
  };

  APIBase = (function() {
    function APIBase(config) {
      this.base = config.baseUrl;
      this.tokens = config.tokens;
    }

    APIBase.prototype._interpolate = function(str, obj) {
      var k, re, v;
      for (k in obj) {
        v = obj[k];
        re = new RegExp("\#{" + k + "}", "g");
        str = str.replace(re, v);
      }
      return str;
    };

    APIBase.prototype._doMapping = function(val, mapping, resource) {
      if (mapping === '$') {
        val = resource;
      } else if (!ud.isString(mapping)) {
        val = mapping;
      } else if (mapping[0] === "#") {
        val = interpolate(mapping.replace(/#/, ''), resource);
      } else if (mapping[0] === "$") {
        val = jp["eval"](resource, mapping);
      } else {
        val = mapping;
      }
      return val;
    };

    APIBase.prototype._getUrl = function(path) {
      return this.base + "/" + path;
    };

    APIBase.prototype._getToken = function(service, cb) {
      if (this.tokens && service in this.tokens) {
        return cb(null, this.tokens[service]);
      } else {
        return cb(null, null);
      }
    };

    APIBase.prototype._handleToken = function(op, req, cb) {
      log.debug('handling token');
      return this._getOperationService(op.id, (function(_this) {
        return function(err, service) {
          if (err) {
            cb(err, null);
            return;
          }
          return _this._getToken(service, function(err, token) {
            log.debug("token:", token);
            if (err) {
              log.error(err);
              cb(err);
              return;
            }
            if (service in oauth) {
              req = oauth[service](token, req);
            }
            return cb(null, req);
          });
        };
      })(this));
    };

    APIBase.prototype._opAPI = {};

    APIBase.prototype._getOperationAPI = function(id, cb) {
      if (id in this._opAPI) {
        cb(null, ud.clone(this._opAPI[id]));
        return;
      }
      return request.get({
        url: this._getUrl("operation/" + id + "/api")
      }, (function(_this) {
        return function(err, res, body) {
          body = JSON.parse(body);
          if (res.statusCode !== 200) {
            cb(body, null);
          } else {
            _this._opAPI[id] = body;
            cb(null, ud.clone(body));
          }
        };
      })(this));
    };

    APIBase.prototype._getOperationService = function(id, cb) {
      return this._getOperationAPI(id, function(err, api) {
        if (err) {
          return cb(err);
        } else {
          return cb(null, api.name.toLowerCase());
        }
      });
    };

    APIBase.prototype.search = function(type, q, page, cb) {
      return request.get({
        url: this._getUrl("search"),
        qs: {
          type: type || "",
          q: q,
          page: page
        }
      }, (function(_this) {
        return function(err, res, body) {
          if (err) {
            cb(err);
            return;
          }
          body = JSON.parse(body);
          if (res.statusCode !== 200) {
            return cb(body, null);
          } else {
            return cb(null, body);
          }
        };
      })(this));
    };

    APIBase.prototype._entityCache = new NodeCache({
      stdTTL: 15 * 60
    });

    APIBase.prototype.getEntity = function(id, brief, cb) {
      var entity, key;
      if (typeof brief === 'function') {
        cb = brief;
        brief = true;
      }
      if (!brief) {
        brief = false;
      }
      key = id + (brief ? ":b" : "");
      entity = this._entityCache.get(key);
      if (entity) {
        cb(null, entity);
        return;
      }
      return request.get({
        url: this._getUrl("" + id),
        qs: {
          brief: brief ? "true" : "false"
        }
      }, (function(_this) {
        return function(err, res, body) {
          body = JSON.parse(body);
          if (res.statusCode !== 200) {
            cb(body, null);
          } else {
            _this._entityCache.set(key, body);
            cb(null, _this._entityCache.get(key));
          }
        };
      })(this));
    };

    APIBase.prototype._apiOpCache = new NodeCache({
      stdTTL: 30 * 60,
      useClones: false
    });

    APIBase.prototype.getAPIOperations = function(id, cb) {
      var ops;
      ops = this._apiOpCache.get(id);
      if (ops) {
        cb(null, ops);
        return;
      }
      return request.get({
        url: this._getUrl("api/" + id + "/operations"),
        json: true
      }, (function(_this) {
        return function(err, res, body) {
          if (res.statusCode !== 200) {
            cb(body, null);
          } else {
            _this._apiOpCache.set(key, body);
            cb(null, body);
          }
        };
      })(this));
    };

    APIBase.prototype.addResourceToOperation = function(oid, rtype, resource, cb) {
      resource._type = 'Resource';
      resource.resourceType = rtype;
      return request.post({
        url: this._getUrl("operation/" + oid + "/resources"),
        json: resource
      }, (function(_this) {
        return function(err, res, body) {
          if (res.statusCode !== 200) {
            cb(body, null);
          } else {
            cb(null, body);
          }
        };
      })(this));
    };

    APIBase.prototype.addRelationship = function(type, id, rel, tid, cb) {
      return request.put({
        url: this._getUrl(type + "/" + id + "/" + rel),
        json: {
          targetId: targetId,
          tid: tid
        }
      }, (function(_this) {
        return function(err, res, body) {
          body = JSON.parse(body);
          if (res.statusCode !== 200) {
            cb(body, null);
          } else {
            cb(null, body);
          }
        };
      })(this));
    };

    APIBase.prototype.getParamRTmapping = function(id, cb) {
      return request.get({
        url: this._getUrl("parameter/" + id + "/resourceType")
      }, function(err, res, body) {
        body = JSON.parse(body);
        if (res.statusCode !== 200) {
          cb(body, null);
        } else {
          cb(null, body);
        }
      });
    };

    APIBase.prototype.createResource = function(type, data, cb) {
      var hs;
      if (data && ud.isArray(data)) {
        data = {
          arrayData: data
        };
      }
      data.resourceType = type;
      hs = {
        'User-Agent': 'APIBase',
        'Content-Type': 'application/json'
      };
      return request.post({
        url: this._getUrl("resource"),
        headers: hs,
        body: JSON.stringify(data)
      }, function(err, res, body) {
        body = JSON.parse(body);
        if (res.statusCode !== 200) {
          cb(body, null);
        } else {
          cb(null, body);
        }
      });
    };

    APIBase.prototype.getResource = function(id, cb) {
      return this.getEntity(id, true, function(err, resp) {
        if (err) {
          cb(err);
          return;
        }
        try {
          log.debug(resp);
          return cb(null, {
            resourcetype: resp.resourcetype,
            data: JSON.parse(resp.data)
          });
        } catch (error) {
          err = error;
          return cb(err);
        }
      });
    };

    APIBase.prototype._resourceToParam = function(pid, rid) {
      var d;
      log.debug("mapping value", rid, pid);
      d = deferred();
      async.parallel([
        (function(_this) {
          return function(cb) {
            return _this.getResource(rid, cb);
          };
        })(this), (function(_this) {
          return function(cb) {
            return _this.getParamRTmapping(pid, cb);
          };
        })(this)
      ], function(err, arg) {
        var i, len, pmap, ref, resource, rt;
        resource = arg[0], pmap = arg[1];
        if (err) {
          log.error(err);
          d.reject(err);
          return;
        }
        log.debug("resource:", resource);
        log.debug("mapping:", pmap);
        ref = resource.resourcetype;
        for (i = 0, len = ref.length; i < len; i++) {
          rt = ref[i];
          if (rt in pmap) {
            d.resolve(jp["eval"](resource.data, pmap[rt]['$']));
            return;
          }
        }
        d.reject(new Error('failed to map resource'));
        return log.error("failed to map resource", resource, pmap);
      });
      return d.promise;
    };

    APIBase.prototype.runRecipe = function(rid, params, cb) {
      return request.post({
        url: this._getUrl("recipe/" + rid + "/run"),
        json: {
          parameters: params
        }
      }, (function(_this) {
        return function(err, res, body) {
          if (res.statusCode !== 200) {
            cb(body, null);
          } else {
            cb(null, res);
          }
        };
      })(this));
    };

    APIBase.prototype.invokeOperation = function(opId, params, resource, map, f_cb) {
      log.debug('invoking...');
      return async.waterfall([
        (function(_this) {
          return function(cb) {
            return _this.getEntity(opId, false, cb);
          };
        })(this), (function(_this) {
          return function(op, cb) {
            if (op._type === 'Task') {
              if (!(typeof task !== "undefined" && task !== null ? task.tasks : void 0) || task.tasks.length > 1) {
                cb(new Error('currently only support single-operation task'), null);
                return;
              }
              log.debug(task.tasks[0]);
              return cb(null, task.tasks[0]);
            } else if (op._type === 'Method') {
              return cb(null, op);
            } else if (op._type === 'Recipe') {
              return cb(null, op);
            } else {
              return cb(new Error('unexpected type ' + op._type));
            }
          };
        })(this), (function(_this) {
          return function(op, cb) {
            if (typeof op === 'object') {
              if ((op != null ? op._type : void 0) !== 'Method' && (op != null ? op._type : void 0) !== 'Recipe') {
                cb(new Error('unexpected type ' + op._type));
                return;
              }
              cb(null, op);
              return;
            }
            return _this.getEntity(op, false, cb);
          };
        })(this), (function(_this) {
          return function(op, cb) {
            if (op._type === 'Recipe') {
              return _this.runRecipe(op.id, params, function(err, res) {
                return cb(err, res, res.body);
              });
            } else {
              return async.waterfall([
                function(cb) {
                  return _this._getOperationAPI(op.id, function(err, api) {
                    return cb(err, op, api);
                  });
                }, function(op, api, cb) {
                  var body, form, headers, qs, val;
                  log.debug('operation: ', op);
                  map = map || {};
                  qs = [];
                  body = {};
                  headers = {};
                  form = {};
                  val = "";
                  log.debug("resource", resource);
                  return async.each(op._relations, function(param, cb) {
                    var t;
                    t = param.target;
                    if (t._type !== 'Parameter') {
                      cb(null);
                      return;
                    }
                    if (params) {
                      val = params[t.name];
                    }
                    if (ud.isString(val) && startsWith(val, 'resource://')) {
                      val = val.replace(/^resource:\/\//, "");
                      log.debug("mapping value:", val);
                      val = _this._resourceToParam(param.target.id, val);
                    } else if (t.name in map) {
                      val = _this._doMapping(val, map[t.name], resource);
                      val = deferred(val);
                    } else {
                      val = deferred(val);
                    }
                    return val.then(function(val) {
                      var e;
                      log.debug('value resolved ' + val);
                      if (ud.has(t, 'locked')) {
                        log.debug('locked parameter');
                        val = null;
                      }
                      if (!val) {
                        if (ud.has(t, 'default')) {
                          val = t["default"];
                        } else {
                          cb(null);
                          return;
                        }
                      }
                      switch (t.location) {
                        case 'header':
                          headers[t.name] = val;
                          break;
                        case 'path':
                          op.path = op.path.replace("{" + t.name + "}", val);
                          break;
                        case 'query':
                          qs[t.name] = val;
                          break;
                        case 'body':
                          body[t.name] = val;
                          if (t.type === 'object') {
                            try {
                              body[t.name] = JSON.parse(val);
                            } catch (error) {
                              e = error;
                              body[t.name] = val;
                            }
                          }
                          break;
                        case 'form':
                          form[t.name] = val;
                          if (t.type === 'file') {
                            form[t.name].value = new Buffer(form[t.name].value, 'base64');
                          }
                      }
                      return cb(null);
                    });
                  }, function(err) {
                    var ctype, hs, k, req, v;
                    log.debug('finished parsing values...', err);
                    if (err) {
                      cb(err);
                      return;
                    }
                    log.debug('preparing request..');
                    hs = {
                      'user-agent': 'APIBase',
                      'content-type': 'application/json'
                    };
                    ctype = 'application/json';
                    for (k in headers) {
                      v = headers[k];
                      hs[k] = v;
                      if (k && k.toLowerCase() === 'content-type') {
                        ctype = v;
                      }
                    }
                    if (!api.baseUrl.endsWith('/')) {
                      api.baseUrl = api.baseUrl + '/';
                    }
                    if (op.path.startsWith('/')) {
                      op.path = op.path.replace(/^\/+/g, '');
                    }
                    req = {
                      method: op.method,
                      url: api.baseUrl + op.path,
                      qs: qs,
                      headers: hs
                    };
                    if (op.method !== 'GET') {
                      switch (ctype) {
                        case 'application/json':
                          req.json = body;
                          break;
                        case 'application/x-www-form-urlencoded':
                          req.form = form;
                          break;
                        case 'multipart/form-data':
                          req.formData = form;
                      }
                    }
                    return _this._handleToken(op, req, cb);
                  });
                }, function(req, cb) {
                  log.debug("making request");
                  return request(req, cb);
                }
              ], cb);
            }
          };
        })(this), (function(_this) {
          return function(res, body, cb) {
            return cb(null, res);
          };
        })(this)
      ], f_cb);
    };

    return APIBase;

  })();

  module.exports = APIBase;

}).call(this);
